# -*- coding: utf-8 -*-
"""Clustering_solution.ipynb

Automatically generated by Colaboratory.

Original file is located at
    https://colab.research.google.com/drive/1lbIiF6TocJYKkrZFMgCEP7cu4YlleqxN

__Clustering 1. DBSCAN Using DBSCAN iterate (for-loop) through different values of min_samples (1 to 10) and epsilon (.05 to .5, in steps of .01) to find clusters in the road-data used in the Lesson and calculate the Silhouette Coeff for min_samples and epsilon. Plot one line plot with the multiple lines generated from the min_samples and epsilon values. Use a 2D array to store the SilCoeff values, one dimension represents min_samples, the other represents epsilon. Expecting a plot of epsilon vs sil_score.__
"""

# Loading Basic Libraries

import numpy as np
import pandas as pd
import matplotlib.pyplot as plt 
import seaborn as sns 

from sklearn.preprocessing import StandardScaler
from sklearn.cluster import DBSCAN
from sklearn.metrics import silhouette_score
from numpy import arange
import warnings
warnings.filterwarnings("ignore")

df= pd.read_csv("3D_spatial_network.txt")

df.columns = [f"col_{str(i)}" for i in range(df.shape[1])]

# shape of the final dataset

print("Our data-set have {} rows and {} columns." .format(df.shape[0], df.shape[1]))

df.head()

# standardization
df_col = df.columns
scaler = StandardScaler().fit(df)
df = scaler.transform(df)
df = pd.DataFrame(df)
df.columns = df_col

min_samples = range(1,11)
epsilons = [round(i,2) for i in  list(arange(0.05, 0.5, 0.1)) + [0.5]]


X = df.values

all_scores = []
for min_sample in min_samples:
    scores = []
    for epsilon in epsilons:
        # print(f"Param= min_sample:{min_sample}, epsilon:{epsilon}")
        cluster = DBSCAN(eps=epsilon, min_samples=min_sample)
        # calculate silouette score here
        score = silhouette_score(X, cluster.fit_predict(X))
        scores.append(score)
        # print(scores)
    all_scores.append(scores)

data=np.array(all_scores)
plt.rcParams["figure.figsize"] = [25, 5]
plt.rcParams["figure.autolayout"] = True
plt.plot(data[:, 0], data[:, 1])

"""__2. Clustering your own data Using your own data, find relevant clusters/groups within your data (repeat the above). If your data is labelled with a class that you are attempting to predict, be sure to not use it in training and clustering. You may use the labels to compare with predictions to show how well the clustering performed using one of the clustering metrics (http://scikit-learn.org/stable/modules/clustering.html#clustering-performance-evaluation). If you don't have labels, use the silhouette coefficient to show performance. Find the optimal fit for your data but you don't need to be as exhaustive as above. Additionally, show the clusters in 2D or 3D plots__"""

df= pd.read_csv("IRIS.csv")
df= df.drop("species",axis=1)
df.head()

min_samples = range(1,4)
epsilons = [round(i,2) for i in  list(arange(0.05, 0.5, 0.1)) + [0.5]]


X = df.values

all_scores = []
min=0
for min_sample in min_samples:
    scores = []
    for epsilon in epsilons:
        # print(f"Param= min_sample:{min_sample}, epsilon:{epsilon}")

        cluster = DBSCAN(eps=epsilon, min_samples=min_sample)
        # calculate silouette score here
        score = silhouette_score(X, cluster.fit_predict(X))
        scores.append(score)
        
        if score>min:
            min = score
            best_cluster = cluster
    all_scores.append(scores)

labels = best_cluster.labels_

# Number of clusters
n_clusters_ = len(set(labels)) - (1 if -1 in labels else 0)
print("Number of Cluster = ", n_clusters_)

# Evaluation
s1 = silhouette_score(df, best_cluster.fit_predict(df))
print("Silhouette score:",s1)

from sklearn.cluster import KMeans

kmeans = KMeans(n_clusters = n_clusters_)

label = kmeans.fit_predict(X)
centroids = kmeans.cluster_centers_
centers = np.array(centroids)
#Getting unique labels
u_labels = np.unique(labels)
 
#plotting the results:

plt.figure(1 , figsize = (15 ,6))
for i in u_labels:
    plt.scatter(X[label == i , 0] , X[label == i , 1] , label = i)


plt.scatter(kmeans.cluster_centers_[:, 0], kmeans.cluster_centers_[:, 1], s=300, c='yellow', label = 'Centroids')

plt.show()

